@@ -727,6 +727,26 @@ roxterm_update_cursor_colour(ROXTermData * roxterm, VteTerminal * vte,
 		vte_terminal_set_color_cursor(vte, NULL);
 }
 
+guint16 extrapolate_chroma(guint16 bg, guint16 fg, double factor)
+{
+	guint ext = (guint) bg + (guint) ((double) (fg - bg) * factor);
+
+	return (guint16) CLAMP(ext, 0, 0xffff);
+}
+
+static const GdkColor *extrapolate_colours(const GdkColor *bg,
+		const GdkColor *fg, double factor)
+{
+	static GdkColor ext;
+
+#define EXTRAPOLATE(c) ext. c = \
+	extrapolate_chroma(bg ? bg->  c : 0, fg ? fg->  c : 0xffff, factor)
+	EXTRAPOLATE(red);
+	EXTRAPOLATE(green);
+	EXTRAPOLATE(blue);
+	return &ext;
+}
+
 static void
 roxterm_apply_colour_scheme(ROXTermData * roxterm, VteTerminal * vte)
 {
@@ -737,55 +757,28 @@ roxterm_apply_colour_scheme(ROXTermData * roxterm, VteTerminal * vte)
 	const GdkColor *background = colour_scheme_get_background_colour
 		(roxterm->colour_scheme, FALSE);
 
-	vte_terminal_set_default_colors(vte);
-
-	ncolours = colour_scheme_get_palette_size(roxterm->colour_scheme);
-	if (ncolours)
-		palette = colour_scheme_get_palette(roxterm->colour_scheme);
-	if (palette)
+	if (!foreground && !background)
 	{
-		vte_terminal_set_colors(vte, foreground, background, palette, ncolours);
+		vte_terminal_set_default_colors(vte);
 	}
 	else
 	{
-		if (foreground)
-			vte_terminal_set_color_foreground(vte, foreground);
-		if (background)
-			vte_terminal_set_color_background(vte, background);
+		ncolours = colour_scheme_get_palette_size(roxterm->colour_scheme);
+		if (ncolours)
+			palette = colour_scheme_get_palette(roxterm->colour_scheme);
+		vte_terminal_set_colors(vte, foreground, background, palette, ncolours);
+		if (!ncolours)
+		{
+			vte_terminal_set_color_bold(vte,
+					extrapolate_colours(background, foreground, 1.2));
+			vte_terminal_set_color_dim(vte,
+					extrapolate_colours(background, foreground, 0.7));
+		}
 	}
 	roxterm_update_cursor_colour(roxterm, vte, FALSE);
 	roxterm_force_redraw(vte);
 }
 
-static void roxterm_update_foreground_colour(ROXTermData * roxterm,
-		VteTerminal * vte)
-{
-	const GdkColor *colour = colour_scheme_get_foreground_colour
-		(roxterm->colour_scheme, FALSE);
-
-	if (colour)
-	{
-		vte_terminal_set_color_foreground(vte, colour);
-		roxterm_force_redraw(vte);
-	}
-	else
-	{
-		roxterm_apply_colour_scheme(roxterm, vte);
-	}
-}
-
-static void roxterm_update_background_colour(ROXTermData * roxterm,
-		VteTerminal * vte)
-{
-	const GdkColor *colour = colour_scheme_get_background_colour
-		(roxterm->colour_scheme, FALSE);
-
-	if (colour)
-		vte_terminal_set_color_background(vte, colour);
-	else
-		roxterm_apply_colour_scheme(roxterm, vte);
-}
-
 static void roxterm_geometry_func(ROXTermData *roxterm,
 		GdkGeometry *geom, GdkWindowHints *hints)
 {
@@ -1856,6 +1849,28 @@ static void roxterm_apply_disable_menu_access(ROXTermData *roxterm)
 			"roxterm");
 }
 
+static void roxterm_apply_title_template(ROXTermData *roxterm)
+{
+	static char *bad_template = NULL;
+	char *tt = options_lookup_string_with_default(roxterm->profile,
+			"title_string", "%s");
+
+	if (tt && !multi_win_validate_title_template(tt))
+	{
+		if (!bad_template || strcmp(bad_template, tt))
+		{
+			dlg_warning(roxterm_get_toplevel(roxterm),
+			  _("'%s' contains invalid %% sequences for a window title string"),
+			  tt);
+			g_free(bad_template);
+			bad_template = tt;
+		}
+		tt = NULL;
+	}
+	multi_win_set_title_template(roxterm->win, tt);
+	g_free(tt);
+}
+	
 static void roxterm_apply_profile(ROXTermData *roxterm, VteTerminal *vte)
 {
 	roxterm_set_select_by_word_chars(roxterm, vte);
@@ -1882,6 +1897,8 @@ static void roxterm_apply_profile(ROXTermData *roxterm, VteTerminal *vte)
 	roxterm_apply_wrap_switch_tab(roxterm);
 
 	roxterm_apply_disable_menu_access(roxterm);
+	
+	roxterm_apply_title_template(roxterm);
 }
 
 static gboolean
@@ -2161,6 +2178,10 @@ static void roxterm_reflect_profile_change(Options * profile, const char *key)
 			mtree = multi_win_get_menu_bar(roxterm->win);
 			menutree_disable_shortcuts(mtree, disable);
 		}
+		else if (!strcmp(key, "title_string"))
+		{
+			roxterm_apply_title_template(roxterm);
+		}
 	}
 }
 
@@ -2231,10 +2252,6 @@ static void roxterm_reflect_colour_change(Options *scheme, const char *key)
 
 		if (!strcmp(key, "cursor"))
 			roxterm_update_cursor_colour(roxterm, vte, TRUE);
-		else if (!strcmp(key, "foreground"))
-			roxterm_update_foreground_colour(roxterm, vte);
-		else if (!strcmp(key, "background"))
-			roxterm_update_background_colour(roxterm, vte);
 		else
 			roxterm_apply_colour_scheme(roxterm, vte);
 	}
